%define afsvers 1.1.1
%define release 1

# Define your particular kernel version:
#	For Linux 2.2:  22
#	For Linux 2.4:	24
#
%define kernvers 22

# This is where to look for kernel-build includes files
# Most likely you want (depending on your situation):
# 	Linux 2.2:
#		kbase = /usr/src/linux-
#		kend = ""
#	Linux 2.4:
#		kbase = /lib/modules/
#		kend = /build
#
%define kbase /usr/src/linux-
%define kend ""

#######################################################################
# You probably don't need to change anything beyond this line

Summary: OpenAFS distributed filesystem
Name: openafs
Version: %{afsvers}
Release: %{kernvers}.%{release}
Copyright: IPL
BuildRoot: /var/tmp/openafs-root
Packager: Derek Atkins <warlord@MIT.EDU>
Group: Networking/Filesystems
Requires: openafs-kernel

Source0: http://www.openafs.org/dl/openafs/${afsvers}/openafs-%{afsvers}-src.tar.gz
Source1: http://www.openafs.org/dl/openafs/${afsvers}/openafs-%{afsvers}-doc.tar.gz
Source2: openafs-ThisCell
Source3: openafs-CellServDB
Source4: openafs-SuidCells
Source5: openafs-cacheinfo
Source6: openafs-afsmodname
Source7: openafs-LICENSE.Sun
Source8: openafs-README
Source9: openafs-Makefile.in

Patch0: openafs-rc-symtab.patch

%description
The AFS distributed filesystem.  AFS is a distributed filesystem
allowing cross-platform sharing of files among multiple computers.
Facilities are provided for access control, authentication, backup and
administrative management.

This package provides common files shared across all the various
OpenAFS packages but are not necessarily tied to a client or server.

%package client
Requires: openafs-kernel, openafs = %{PACKAGE_VERSION}
Summary: OpenAFS Filesystem Client
Group: Networking/Filesystem

%description client
The AFS distributed filesystem.  AFS is a distributed filesystem
allowing cross-platform sharing of files among multiple computers.
Facilities are provided for access control, authentication, backup and
administrative management.

This package provides basic client support to mount and manipulate
AFS.

%package server
Requires: openafs-kernel, openafs = %{PACKAGE_VERSION}
Summary: OpenAFS Filesystem Server
Group: Networking/Filesystems

%description server
The AFS distributed filesystem.  AFS is a distributed filesystem
allowing cross-platform sharing of files among multiple computers.
Facilities are provided for access control, authentication, backup and
administrative management.

This package provides basic server support to host files in an AFS
Cell.

%package devel
Summary: OpenAFS Development Libraries and Headers
Group: Development/Filesystems

%description devel
The AFS distributed filesystem.  AFS is a distributed filesystem
allowing cross-platform sharing of files among multiple computers.
Facilities are provided for access control, authentication, backup and
administrative management.

This package provides static development libraries and headers needed
to compile AFS applications.  Note: AFS currently does not provide
shared libraries.

%package kernel
Summary: OpenAFS Kernel Module(s)
Requires: openafs = %{PACKAGE_VERSION}
Group: Networking/Filesystems

%description kernel
The AFS distributed filesystem.  AFS is a distributed filesystem
allowing cross-platform sharing of files among multiple computers.
Facilities are provided for access control, authentication, backup and
administrative management.

This package provides precompiled AFS kernel modules for various
kernels.

%package kernel-source
Summary: OpenAFS Kernel Module source tree
Group: Networking/Filesystems

%description kernel-source
The AFS distributed filesystem.  AFS is a distributed filesystem
allowing cross-platform sharing of files among multiple computers.
Facilities are provided for access control, authentication, backup and
administrative management.

This package provides the source code to build your own AFS kernel
module.

%prep
%setup -q -b 1

%patch0 -p0

###
### build
###
%build

%ifarch i386 i486 i586 i686 athlon
sysbase=i386
%else
sysbase=%{_arch}
%endif
sysname=${sysbase}_linux%{kernvers}

if [ %{kernvers} = 22 ]; then
   kv='2\.2\.'
elif [ %{kernvers} = 24 ]; then
   kv='2\.4\.'
else
   echo "I don't know how to build $sysname"
   exit 1
fi

%ifarch i386 i486 i586 i686 athlon
archlist="i386 i586 i686 athlon"
%else
archlist=${sysbase}
%endif

#
# PrintDefine var value statements file
#
PrintDefine() {
    case $3 in
    *ifn*)
	echo "#ifndef $1" >> $4
	;;
    esac
    case $3 in
    *def*)
	echo "#define $1 $2" >> $4
	;;
    esac
    case $3 in
    *end*)
	echo "#endif" >> $4
	;;
    esac
    case $3 in
    *und*)
	echo "#undef $1" >> $4
	;;
    esac
    case $3 in
    *inc*)
	echo "#include $1" >> $4
	;;
    esac


    case $3 in
    *nl*)
	echo "" >> $4
	;;
    esac
}

# PrintRedhatKernelFix arch mp file
PrintRedhatKernelFix() {
    arch="$1"
    if [ "$2" = "MP" ]; then
	smp=1
	up=0
    else
	smp=0
	up=1
    fi
    file="$3"

    rm -f $file
    touch $file

    PrintDefine "REDHAT_FIX_H" "" ifn,def,nl $file

    PrintDefine "__BOOT_KERNEL_ENTERPRISE" 0 ifn,def,end,nl $file
    PrintDefine "__BOOT_KERNEL_SMP" $smp ifn,def,end,nl $file
    PrintDefine "__BOOT_KERNEL_UP" $up ifn,def,end,nl $file

    PrintDefine \"/boot/kernel.h\" "" inc,nl $file	# include file

    for ar in $archlist ; do
	if [ "$ar" = "$arch" ]; then
	    PrintDefine "__BOOT_KERNEL_$ar" "1" ifn,def,end $file
	else
	    PrintDefine "__BOOT_KERNEL_$ar" "" und $file	# undef
        fi
    done
    echo "" >> $file

    PrintDefine "" "" end $file
}

# Pick up all the 'appropriate' kernels
kvers=`ls -d %{kbase}* | sed 's^%{kbase}^^g' | grep $kv`

# Choose the last one for now.. It doesn't really matter, really.
hdrdir=`ls -d %{kbase}*%{kend} | grep $kv | tail -1`

# Configure AFS
./configure --enable-redhat-buildsys --with-afs-sysname=${sysname} \
	--with-linux-kernel-headers=$hdrdir

# Build the user-space AFS stuff
make all_nolibafs

# Now build all the kernel modules
for vers in $kvers ; do

  # Reconfigure sources for this kernel version, to catch various
  # kernel params in the configure script.  Yes. this takes more time,
  # but it's worth it in the long run..  But first remove config.cache
  # to be sure we get a clean configuration.
  rm -f config.cache
  ./configure --enable-redhat-buildsys --with-afs-sysname=${sysname} \
	--with-linux-kernel-headers=%{kbase}$vers%{kend}

  # Now build both SP and MP versions...
  for mp in SP MP ; do

    # ... for all appropriate 'architectures'...
    if [ %{kernvers} = 22 ]; then
	# For 2.2 kernels, just do SMP and UP kernels for this version

	PrintRedhatKernelFix ${sysbase} $mp src/config/redhat-fix.h

	make only_libafs LOCAL_SMP_DEF=-DREDHAT_FIX MPS=$mp

    elif [ %{kernvers} = 24 ]; then
	# For 2.4 kernels, need to build modules for each architecture!

	for arch in $archlist ; do

	    PrintRedhatKernelFix $arch $mp src/config/redhat-fix.h

	    make only_libafs LOCAL_SMP_DEF=-DREDHAT_FIX \
		LINUX_MODULE_NAME="-$arch" MPS=$mp
	done

    else	    
	echo "I don't know how to build $sysname"
	exit 1
    fi
  done
done

rm -f src/config/redhat-fix.h

###
### install
###
%install

%ifarch i386 i486 i586 i686 athlon
sysbase=i386
%else
sysbase=%{_arch}
%endif
sysname=${sysbase}_linux%{kernvers}

# Build install tree
mkdir -p $RPM_BUILD_ROOT/afs
mkdir -p $RPM_BUILD_ROOT/usr/sbin
mkdir -p $RPM_BUILD_ROOT/etc/sysconfig
mkdir -p $RPM_BUILD_ROOT/etc/rc.d/init.d
mkdir -p $RPM_BUILD_ROOT/etc/openafs
mkdir -p $RPM_BUILD_ROOT/lib/security
mkdir -p $RPM_BUILD_ROOT/usr/afs/logs
mkdir -p $RPM_BUILD_ROOT/usr/vice/etc
mkdir -p $RPM_BUILD_ROOT/usr/vice/cache
chmod 700 $RPM_BUILD_ROOT/usr/vice/cache

# Copy files from dest to the appropriate places in BuildRoot
tar cf - -C ${sysname}/dest bin include lib | tar xf - -C $RPM_BUILD_ROOT/usr
tar cf - -C ${sysname}/dest/etc . | tar xf - -C $RPM_BUILD_ROOT/usr/sbin
tar cf - -C ${sysname}/dest/root.server/usr/afs bin | tar xf - -C $RPM_BUILD_ROOT/usr/afs
tar cf - -C ${sysname}/dest/root.client/usr/vice/etc afsd modload | tar xf - -C $RPM_BUILD_ROOT/usr/vice/etc

# Link kpasswd to kapasswd
ln -f $RPM_BUILD_ROOT/usr/bin/kpasswd $RPM_BUILD_ROOT/usr/bin/kapasswd

# Copy root.client config files
install -m 755 ${sysname}/dest/root.client/usr/vice/etc/afs.conf $RPM_BUILD_ROOT/etc/sysconfig/afs
install -m 755 ${sysname}/dest/root.client/usr/vice/etc/afs.rc $RPM_BUILD_ROOT/etc/rc.d/init.d/afs

# Copy PAM modules
install -m 755 ${sysname}/dest/lib/pam* $RPM_BUILD_ROOT/lib/security

# Populate /usr/vice/etc
uve=$RPM_BUILD_ROOT/usr/vice/etc
install -p -m 644 $RPM_SOURCE_DIR/openafs-CellServDB $uve/CellServDB
install -p -m 644 $RPM_SOURCE_DIR/openafs-SuidCells $uve/SuidCells
install -p -m 644 $RPM_SOURCE_DIR/openafs-ThisCell $uve/ThisCell
install -p -m 644 $RPM_SOURCE_DIR/openafs-cacheinfo $uve/cacheinfo
install -p -m 755 $RPM_SOURCE_DIR/openafs-afsmodname $uve/afsmodname

#
# Build the SymTable
symtable=$RPM_BUILD_ROOT/usr/vice/etc/modload/SymTable
rm -f $symtable
echo "# SymTable, automatically generated" > $symtable
echo "# symbol	version	cpu	module" >> $symtable
echo "" >> $symtable

for mod in $RPM_BUILD_ROOT/usr/vice/etc/modload/libafs*.o ; do
    sym=`nm $mod | grep iget | sed -e 's/^[ 	]*U //'`
    sym2="`echo $sym | sed -e 's/\(.*\)_R/\1	/'`"
    f=`basename $mod | sed -e 's/^libafs-//' -e 's/.o$//'`
    if [ "$sym" = "$sym2" ]; then
	echo "No Symbol Version found: $sym"
    else
	echo "$sym2	-	$f" >> $symtable
    fi
done

#
# install kernel-source
#

# Install the kernel module source tree
mkdir -p $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/src
tar cf - src/libafs/Makefile* src/libafs/afs src/libafs/afsint src/libafs/rx \
    src/libafs/config \
    acconfig.h aclocal.m4 config.guess config.sub configure configure.in \
    install-sh mkinstalldirs regen.sh src/config \
  | tar xf - -C $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}

# Make files that configure cares about
mkdir -p $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/src/pam
touch $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/src/pam/Makefile.in

mkdir -p $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/src/afsd
touch $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/src/afsd/afs.ppc_darwin.plist.in

# Copy Makefile.common
#   copy by hand, as they are usually symlinks
install -m 644 src/libafs/Makefile.common $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/src/libafs/

# Next, copy the LICENSE Files, README, and top-level Makefile.in
install -m 644 src/LICENSE $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/LICENSE.IBM
install -m 644 $RPM_SOURCE_DIR/openafs-LICENSE.Sun $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/LICENSE.Sun
install -m 644 $RPM_SOURCE_DIR/openafs-README $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/README
install -m 644 $RPM_SOURCE_DIR/openafs-Makefile.in $RPM_BUILD_ROOT/usr/src/openafs-kernel-%{afsvers}/Makefile.in

#
# Install DOCUMENTATION
#

# Build the DOC directory
mkdir -p $RPM_BUILD_ROOT/usr/doc/openafs-%{afsvers}
tar cf - -C doc LICENSE html pdf | \
    tar xf - -C $RPM_BUILD_ROOT/usr/doc/openafs-%{afsvers}


###
### clean
###
%clean
[ "$RPM_BUILD_ROOT" != "/" ] && rm -fr $RPM_BUILD_ROOT


###
### scripts
###
%post
chkconfig --add afs

%post client
echo
echo The AFS cache is configured for 100 MB. Edit the 
echo /usr/vice/etc/cacheinfo file to change this before
echo running AFS for the first time. You should also
echo set your home cell in /usr/vice/etc/ThisCell.
echo
echo Also, you may want to edit /etc/pam.d/login and 
echo possibly others there to get an AFS token on login.
echo Put the line:
echo 
echo    auth	   sufficient   /lib/security/pam_afs.so try_first_pass ignore_root
echo
echo before the one for pwdb.
echo

%post server
if [ -f /etc/sysconfig/afs ]; then
	srv=`grep ^AFS_SERVER /etc/sysconfig/afs | sed 's/^AFS_SERVER[\s]*=[\s]*//'`
	if [ $srv eq "on" ]; then
		exit 0
	fi
fi

echo
echo Be sure to edit /etc/sysconfig/afs and turn AFS_SERVER on
echo

%preun
if [ $1 = 0 ] ; then
        /etc/rc.d/init.d/afs stop
        chkconfig --del afs
fi

###
### file lists
###
%files
%defattr(-,root,root)
%dir /usr/vice/etc/modload
%config /etc/sysconfig/afs
%doc /usr/doc/openafs-%{afsvers}
/etc/rc.d/init.d/afs
/usr/bin/afsmonitor
/usr/bin/bos
/usr/bin/fs
/usr/bin/kapasswd
/usr/bin/klog
/usr/bin/klog.krb
/usr/bin/pagsh
/usr/bin/pts
/usr/bin/scout
/usr/bin/sys
/usr/bin/tokens
/usr/bin/tokens.krb
/usr/bin/udebug
/usr/bin/unlog
/usr/sbin/backup
/usr/sbin/butc
/usr/sbin/fstrace
/usr/sbin/kas
/usr/sbin/read_tape
/usr/sbin/restorevol
/usr/sbin/rxdebug
/usr/sbin/uss
/usr/sbin/vos

%files client
%defattr(-,root,root)
%dir /afs
%dir /usr/vice/cache
%config /usr/vice/etc/CellServDB
%config /usr/vice/etc/SuidCells
%config /usr/vice/etc/ThisCell
%config /usr/vice/etc/cacheinfo
/usr/bin/cmdebug
/usr/vice/etc/afsd
/lib/security/pam_afs.krb.so.1
/lib/security/pam_afs.so.1

%files server
%defattr(-,root,root)
%dir /usr/afs
%dir /usr/afs/bin
%dir /usr/afs/logs
/usr/afs/bin/bosserver
/usr/afs/bin/buserver
/usr/afs/bin/fileserver
# Should we support KAServer?
/usr/afs/bin/kaserver
/usr/afs/bin/kpwvalid
/usr/afs/bin/pt_util
/usr/afs/bin/ptserver
/usr/afs/bin/salvager
/usr/afs/bin/upclient
/usr/afs/bin/upserver
/usr/afs/bin/vlserver
/usr/afs/bin/volinfo
/usr/afs/bin/volserver
/usr/sbin/prdb_check
/usr/sbin/vldb_check
/usr/sbin/vldb_convert

%files devel
%defattr(-,root,root)
/usr/bin/rxgen
/usr/include/afs
/usr/include/des.h
/usr/include/des_conf.h
/usr/include/des_odd.h
/usr/include/lock.h
/usr/include/lwp.h
/usr/include/mit-cpyright.h
/usr/include/potpourri.h
/usr/include/preempt.h
/usr/include/rx
/usr/include/timer.h
/usr/include/ubik.h
/usr/include/ubik_int.h
/usr/lib/afs
/usr/lib/libafsauthent.a
/usr/lib/libafsrpc.a
/usr/lib/libdes.a
/usr/lib/liblwp.a
/usr/lib/librx.a
/usr/lib/librxkad.a
/usr/lib/librxstat.a
/usr/lib/libubik.a

%files kernel
%defattr(-,root,root)
/usr/vice/etc/afsmodname
/usr/vice/etc/modload/libafs*.o
/usr/vice/etc/modload/SymTable

%files kernel-source
%defattr(-,root,root)
/usr/src/openafs-kernel-%{afsvers}/LICENSE.IBM
/usr/src/openafs-kernel-%{afsvers}/LICENSE.Sun
/usr/src/openafs-kernel-%{afsvers}/README
/usr/src/openafs-kernel-%{afsvers}/Makefile.in
/usr/src/openafs-kernel-%{afsvers}/acconfig.h
/usr/src/openafs-kernel-%{afsvers}/aclocal.m4
/usr/src/openafs-kernel-%{afsvers}/config.guess
/usr/src/openafs-kernel-%{afsvers}/config.sub
/usr/src/openafs-kernel-%{afsvers}/configure
/usr/src/openafs-kernel-%{afsvers}/configure.in
/usr/src/openafs-kernel-%{afsvers}/install-sh
/usr/src/openafs-kernel-%{afsvers}/mkinstalldirs
/usr/src/openafs-kernel-%{afsvers}/regen.sh
/usr/src/openafs-kernel-%{afsvers}/src
